

import java.net.URI;
import java.util.Arrays;
import java.util.List;

import javax.tools.Diagnostic;
import javax.tools.DiagnosticCollector;
import javax.tools.JavaCompiler;
import javax.tools.JavaFileObject;
import javax.tools.SimpleJavaFileObject;
import javax.tools.ToolProvider;

public class Bytecode {

	private static class SourceFile extends SimpleJavaFileObject {
		final String code;

		SourceFile(String name, String code) {
			super(URI.create("string:///" + name.replace('.', '/')
					+ Kind.SOURCE.extension), Kind.SOURCE);
			this.code = code;
		}

		@Override
		public CharSequence getCharContent(boolean ignoreEncodingErrors) {
			return code;
		}

	}

	/**
	 * Generates a .class file from the Java source code which is given in
	 * source. The name of class in the source code should match the one given
	 * in the parameter className
	 * 
	 * @param className
	 *            The name of the generated class
	 * @param source
	 *            The source code containing the definition of the generated
	 *            class
	 */
	static void generateFromSource(String className, String source) {
		JavaCompiler compiler = ToolProvider.getSystemJavaCompiler();
		List<? extends JavaFileObject> jfiles = Arrays
				.asList(new Bytecode.SourceFile(className, source));

		DiagnosticCollector<JavaFileObject> diagnostics = new DiagnosticCollector<JavaFileObject>();

		boolean succeeded = compiler.getTask(null, null, diagnostics, null,
				null, jfiles).call();

		if (!succeeded) {
			for (Diagnostic<?> diagnostic : diagnostics.getDiagnostics()) {
				System.out.format("Error on line %d in %s",
						diagnostic.getLineNumber(), diagnostic);
			}
		}
	}

}
