import java.io.IOException;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Arrays;
import java.util.Scanner;

public class GriddlerTest {

	public static void printPicture(int[][] picture) {
		for (int[] row : picture) {
			printRow(row);
			System.out.println("");
		}
	}

	private static void printRow(int[] row) {
		for (int pixel : row)
			if (pixel == 0)
				System.out.print(" ");
			else
				System.out.print("X");
	}

	public static int[][] solveFile(Path path) throws IOException {
		try (Scanner s = new Scanner(path)) {
			int[][] columns = null;
			int[][] rows = null;
			int index = 0;

			while (s.hasNextLine()) {
				String line = s.nextLine();

				if (line.isEmpty()) {
					if (rows == null)
						columns[index++] = new int[0];
					else
						rows[index++] = new int[0];

					continue;
				}

				String[] tokens = line.split(" ");

				switch (tokens[0]) {
				case "columns":
					columns = new int[Integer.parseInt(tokens[1])][];
					index = 0;
					continue;
				case "rows":
					rows = new int[Integer.parseInt(tokens[1])][];
					index = 0;
					continue;
				}

				if (rows == null)
					itemFromTokens(columns, index, tokens);
				else
					itemFromTokens(rows, index, tokens);

				++index;
			}
			return Griddler.solve(columns, rows);
		}
	}

	private static void itemFromTokens(int[][] columns, int index,
			String[] tokens) {
		columns[index] = new int[tokens.length];
		for (int i = 0; i < tokens.length; ++i)
			columns[index][i] = Integer.parseInt(tokens[i]);
	}

	public static void main(String[] args) {
		boolean failed = false;
		int[] lengths1 = new int[] { 2, 3 };
		int[] firstPattern1 = Griddler.firstPattern(lengths1);
		if (!Arrays.equals(firstPattern1, new int[] { 0, 3 })) {
			System.out.println("First pattern failed");
			failed = true;
		}

		int[] firstPattern2 = Griddler.firstPattern(new int[] { 6, 1, 3 });
		if (!Arrays.equals(firstPattern2, new int[] { 0, 7, 9 })) {
			System.out.println("Second pattern failed");
			failed = true;
		}

		boolean b = false;
		int[][] expected = { { 0, 4 }, { 0, 5 }, { 1, 4 }, { 1, 5 }, { 2, 5 } };
		for (int i = 0; i < expected.length; ++i) {
			b = Griddler.nextPattern(firstPattern1, lengths1, 8);
			if (!b || !Arrays.equals(firstPattern1, expected[i])) {
				System.out.println("nextPattern failed (1)");
				failed = true;
			}
		}

		b = Griddler.nextPattern(firstPattern1, lengths1, 8);
		if (b) {
			System.out.println("nextPattern failed -- should return false");
			failed = true;
		}

		int[][] render_param = { { 0, 3 }, { 1, 5 } };
		int[][] render_expected = { { 1, 1, 0, 1, 1, 1, 0, 0 },
				{ 0, 1, 1, 0, 0, 1, 1, 1 } };
		int[] row = new int[8];
		for (int i = 0; i < render_param.length; ++i) {
			Griddler.renderPatternInRow(render_param[i], lengths1, row);
			if (!Arrays.equals(row, render_expected[i])) {
				System.out.println("renderPattern in row failed (1)");
				failed = true;
			}
		}

		int[][] picture = { { 1, 1, 0, 1, 1, 1, 0, 0 },
				{ 1, 1, 0, 0, 1, 1, 1, 0 }, { 1, 1, 0, 0, 0, 1, 1, 1 },
				{ 0, 1, 1, 0, 1, 1, 1, 0 }, { 0, 0, 1, 1, 0, 1, 1, 1 } };

		int[][] column_defs = { { 2 }, { 2 }, { 4 }, { 4 } };
		int[] rows = { 2, 3, 5, 5 };
		int[] column_index = { 0, 0, 1, 2 };
		boolean[] violation_results = { false, true, false, false };

		for (int i = 0; i < column_defs.length; ++i)
			if (Griddler.isColumnViolated(picture, column_defs[i], rows[i],
					column_index[i]) != violation_results[i]) {
				System.out.println("isColumnViolated failed on example (" + i
						+ ")");
				failed = true;
			}

		int[][] solve_columns = { { 3 }, { 4 }, { 2 }, { 1, 1 }, { 2, 1 },
				{ 5 }, { 4 }, { 1, 1 } };
		int[][] solve_rows = { { 2, 3 }, { 2, 3 }, { 2, 3 }, { 2, 3 }, { 2, 3 } };
		int[][] result = Griddler.solve(solve_columns, solve_rows);

		if (!areEqualMatrices(picture, result)) {
			System.out.println("solve failed");
			failed = true;
		}

		if (!failed) {
			try {
				String[] files = { "clover.txt", "dino.txt", "girl.txt",
						"rose.txt", "tnt.txt" };
				for (String filename : files) {
					System.out.println(filename);
					printPicture(solveFile(Paths.get("resources", filename)));
					System.out.println("");
				}
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

			System.out.println("Q2 Passed");
		}
	}

	public static boolean areEqualArrays(int[] expected, int[] actual) {
		if (expected == null || actual == null)
			return expected == null && actual == null;

		if (expected.length != actual.length)
			return false;

		for (int i = 0; i < actual.length; i++)
			if (expected[i] != actual[i])
				return false;

		return true;
	}

	public static boolean areEqualMatrices(int[][] expected, int[][] actual) {
		if (expected == null || actual == null)
			return expected == null && actual == null;

		if (expected.length != actual.length)
			return false;

		for (int i = 0; i < expected.length; ++i)
			if (!areEqualArrays(expected[i], actual[i]))
				return false;

		return true;
	}
}
