

import java.io.StringBufferInputStream;

@SuppressWarnings("deprecation")
public class InterpreterTest {

	private static double EPSILON = 0.001;
	private static boolean failed = false;

	private static boolean equal(double a, double b) {
		return Math.abs(a - b) < EPSILON;
	}

	private static boolean notEqual(double a, double b) {
		return !equal(a, b);
	}

	public static void main(String[] args) {
		pushAndVerify(1.1, 1);
		pushAndVerify(3.0, 2);
		pushAndVerify(5.4, 3);

		popAndVerify(5.4, 2);
		popAndVerify(3.0, 1);
		popAndVerify(1.1, 0);

		runCommandAndVerify("2.2", 2.2, 1);
		runCommandAndVerify("2", 2.0, 2);
		runCommandAndVerify("+", 4.2, 1);
		runCommandAndVerify("0.5", 0.5, 2);
		runCommandAndVerify("*", 2.1, 1);
		runCommandAndVerify("2", 2.0, 2);
		runCommandAndVerify("swap", 2.1, 2);
		runCommandAndVerify("**", 4.41, 1);
		runCommandAndVerify("PI", 3.1415926, 2);
		runCommandAndVerify("E", 2.71828, 3);

		popAndVerify(2.71828, 2);
		popAndVerify(3.1415926, 1);
		popAndVerify(4.41, 0);

		runScriptAndVerify(new double[] {}, "args 0\n2.2\n2\n+",
				new double[] { 4.2 });
		runScriptAndVerify(new double[] { 10 }, "args 1\n2\nPI\n*\n*\n1\nswap",
				new double[] { 1.0, 62.8318 });
		
		runFailedScript(new double[] {}, "2.0");
		runFailedScript(new double[] {}, "args 1\nPI");
		runFailedScript(new double[] { 1.0 }, "args 2\nPI");

		if (!failed)
			System.out.println("Q1a Passed");
	}

	private static void runFailedScript(double[] arguments, String script) {
		pushArguments(arguments);

		System.setIn(new StringBufferInputStream(script));

		if (Interpreter.interpret())
			System.out.println("Interpreter returns true on failed conditions");

		while (Interpreter.size() > 0)
			Interpreter.pop();
	}

	private static void runScriptAndVerify(double[] arguments, String script,
			double[] result) {
		pushArguments(arguments);

		System.setIn(new StringBufferInputStream(script));

		if (!Interpreter.interpret())
			System.out.println("Failed to interpret script correctly");

		for (int i = result.length - 1; i >= 0; --i)
			if (notEqual(Interpreter.pop(), result[i]))
				System.out.println("Failed to verify stack state for script: "
						+ script.replaceAll("\n", "\\\\n"));
	}

	private static void pushArguments(double[] arguments) {
		int size = 0;
		verifySize(size);

		for (double arg : arguments)
			pushAndVerify(arg, ++size);
	}

	private static void runCommandAndVerify(String cmd, double top, int size) {
		double ret = Interpreter.runCommand(cmd);

		if (notEqual(ret, top)) {
			System.out.println("Failed to verify command (" + cmd + ")");
			failed = true;
		}

		verifySize(size);
	}

	private static void popAndVerify(double value, int size) {
		double top = Interpreter.pop();

		if (notEqual(top, value)) {
			System.out.println("Failed to verify pop (" + value + ")");
			failed = true;
		}

		verifySize(size);
	}

	private static void pushAndVerify(double value, int size) {
		Interpreter.push(value);
		if (notEqual(Interpreter.top(), value)) {
			System.out.println("Failed to verify push ("
					+ String.valueOf(value) + ")");
			failed = true;
		}

		verifySize(size);
	}

	private static void verifySize(int size) {
		if (Interpreter.size() != size) {
			System.out.println("Stack size should be " + size);
			failed = true;
		}
	}
}
